// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default


#ifndef QTCANVASPAINTERGLOBAL_H
#define QTCANVASPAINTERGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtCanvasPainter/qtcanvaspainterexports.h>

#ifdef Q_QDOC
#define Q_CANVASPAINTER_EXPORT
#endif

#endif
